% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( cytoplasm )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [ADP_cytoplasm , ATP_cytoplasm , AMP_cytoplasm , Pi_cytoplasm ]
%
%
% Free Parameters:
% [k1_ATPASE ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
%% LIST OF STATE VARIABLES
% 1 ADP_cytoplasm
% 2 ATP_cytoplasm
% 3 AMP_cytoplasm
% 4 Pi_cytoplasm
% 5 h_cytoplasm
% 6 m_cytoplasm
% 7 k_cytoplasm

% PARTIAL VOLUME FRACTIONS
VWater_cytoplasm = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_cytoplasm = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
ADP_cytoplasm = x(1);
ATP_cytoplasm = x(2);
AMP_cytoplasm = x(3);
Pi_cytoplasm = x(4);
% Concentrations of H, Mg, and K
h_cytoplasm = x(5);
m_cytoplasm = x(6);
k_cytoplasm = x(7);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% ADP_cytoplasm
Kh(1) = 4.1856568564882886504828268456335216285424394300207e-07;
Km(1) = 0.0008821191357550335187392986924237447965424507856369;
Kk(1) = 0.13114858875318427533507303905935259535908699035645;
% ATP_cytoplasm
Kh(2) = 2.7990983755242071437057782383461823627612830023281e-07;
Km(2) = 0.00010815244062499600717370401570249782707833219319582;
Kk(2) = 0.097055055483606045285682739631738513708114624023438;
% AMP_cytoplasm
Kh(3) = 6.4641151032791652445184471878492082907996518770233e-07;
Km(3) = 0.021233332579732779737957670818104816135019063949585;
Kk(3) = 0.23906119225358615776322324109059991315007209777832;
% Pi_cytoplasm
Kh(4) = 2.1306351186738022140046010417802069270010179025121e-07;
Km(4) = 0.032137949367841568693027198833078728057444095611572;
Kk(4) = 0.37888645618434613204783545370446518063545227050781;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_cytoplasm/Kh(1) + m_cytoplasm/Km(1) + k_cytoplasm/Kk(1);
P( 2 ) = 1  + h_cytoplasm/Kh(2) + m_cytoplasm/Km(2) + k_cytoplasm/Kk(2);
P( 3 ) = 1  + h_cytoplasm/Kh(3) + m_cytoplasm/Km(3) + k_cytoplasm/Kk(3);
P( 4 ) = 1  + h_cytoplasm/Kh(4) + m_cytoplasm/Km(4) + k_cytoplasm/Kk(4);

%% THERMODYNAMIC EQUATIONS
DGro_AK =2.2617;
DGro_ATPASE =4.5083;

Keq_AK_cytoplasm = exp(-DGro_AK/RT)/P(1)^2*P(2)*P(3);
Keq_ATPASE_cytoplasm = exp(-DGro_ATPASE/RT)*P(1)/P(2)*P(4)/h_cytoplasm;

%% FLUX EQUATIONS
%AK_cytoplasm
x_AK=1e7;
J_AK_cytoplasm=x_AK*(Keq_AK_cytoplasm*ADP_cytoplasm*ADP_cytoplasm-AMP_cytoplasm*ATP_cytoplasm);
%Modified,06/11/08
%ATPASE_cytoplasm
k1_ATPASE=par(1);
J_ATPASE_cytoplasm=k1_ATPASE*(ATP_cytoplasm-ADP_cytoplasm*Pi_cytoplasm/Keq_ATPASE_cytoplasm);

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 2*J_AK_cytoplasm + 1*J_ATPASE_cytoplasm ) / VWater_cytoplasm; % ADP_cytoplasm
f(2,:) = ( 0  + 1*J_AK_cytoplasm - 1*J_ATPASE_cytoplasm ) / VWater_cytoplasm; % ATP_cytoplasm
f(3,:) = ( 0  + 1*J_AK_cytoplasm ) / VWater_cytoplasm; % AMP_cytoplasm
f(4,:) = ( 0  + 1*J_ATPASE_cytoplasm ) / VWater_cytoplasm; % Pi_cytoplasm

%% ION EQUATIONS
% COMPARTMENT cytoplasm:
ii = [1  2  3  4]; % Indices of SVs in compartment cytoplasm
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_cytoplasm*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_cytoplasm*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_cytoplasm./Km(ii)+k_cytoplasm./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_cytoplasm*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_cytoplasm*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_cytoplasm./Kh(ii)+k_cytoplasm./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_cytoplasm*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_cytoplasm*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_cytoplasm./Kh(ii)+m_cytoplasm./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 + 0*J_AK_cytoplasm + 1*J_ATPASE_cytoplasm) / VWater_cytoplasm;
J_M = (0) / VWater_cytoplasm;
J_K = (0) / VWater_cytoplasm;
Phi_H = J_H - sum( h_cytoplasm*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_cytoplasm*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_cytoplasm*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_cytoplasm/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(5,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(6,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(7,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_AK_cytoplasm J_ATPASE_cytoplasm];
